#!/usr/bin/python3
# -*- coding: utf-8 -*-
#
# Copyright 2016 Collabora Ltd.
#
# SPDX-License-Identifier: MPL-2.0
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import glob
import sys
import re

COMMIT_GUIDE="""

It seems something isn't right with the commit message. Having a good
commit message is very important for both for reviewers and for people looking
at history later on - code and commits are read many more times than they are
written. Please ensure that your messages follow the convention.

Good guides for writing commit messages can be found on:
 http://tbaggery.com/2008/04/19/a-note-about-git-commit-messages.html
 http://chris.beams.io/posts/git-commit/

More general guidelines for Apertis can be found on:
  https://wiki.apertis.org/Guidelines/Version_control
  https://wiki.apertis.org/Guidelines/Contribution_process
"""
SOB_PREFIX = "Signed-off-by: "
issuefile = None


def add_issue(text):
    global issuefile
    if not issuefile:
        issuefile = open("issues.txt", "w")
    t = "Issue: {}\n".format(text)
    print(t, end="")
    issuefile.write(t)


def valid_mail(address):
    # Mail should be Full Name <full.name@badger.snake>
    # Catch issues when people do:
    #   * unix user rather then name
    #   * Only first name, not full name
    #   * Incorrect spacing
    m = re.compile("[^\s\d]+ ([^\s\d]+ )+<[\w.]+@[\w.]+>")
    return m.match(address)

commits = glob.glob("*.patch")
assert len(commits) == 1
commit = commits[0]

print("* Checking", commit)

inheader = True
header = {}
body = []
for line in open(commit).readlines():
    line = line.rstrip()
    if inheader:
        if not line:
            inheader = False
            continue
        d = line.split(":", 1)
        header[d[0]] = d[1].strip()
    else:
        if line.startswith("diff "):
            break
        body.append(line)

# Check if the author/from is valid
if not valid_mail(header['From']):
    add_issue("Invalid from: {}".format(header['From']))


# Subject should start with RFC or there should be a signed-off-by line
if header['Subject'].startswith("RFC:"):
    print("* RFC style patch")
else:
    sob = None
    for line in body:
        if line.startswith(SOB_PREFIX):
            sob = line[len(SOB_PREFIX):].strip()
            if valid_mail(sob):
                print("* Valid Signed-off-by:", sob)
            else:
                add_issue("Signed-off-by invalid: {}".format(sob))
    if not sob:
        add_issue("Neither RFC or Signed-off-by found")

# No Reviewed-by or reviewers should be present in the body, that's added when
# landing based on information from phabricator.
for line in body:
    forbidden = ["Reviewed-by:", "Reviewers"]
    for f in forbidden:
        if line.startswith(f):
            add_issue("Forbidden tag: {}".format(line))

# Subject too long
if len(header['Subject']) > 75:
    add_issue("Subject too long ({} chars, <= 72 recommended): {}".format(
        len(header['Subject']), header['Subject']))

# Line in body too long
for line in body:
    if len(line) <= 75:
        continue
    if line.startswith(SOB_PREFIX):
        continue

    if len(line.split()) == 1:
        # bail out if the line contains a single long word (for instance an URL)
        continue

    add_issue("Body line too long ({} chars, <= 72 recommended): {}".format(
        len(line), line))


if issuefile is not None:
    issuefile.write(COMMIT_GUIDE)
    sys.exit(1)
